# frozen_string_literal: true

require 'redis-client'
require 'digest/sha2'

begin
  require 'hiredis-client'
  RedisClient.default_driver = :hiredis
rescue LoadError
  Jekyll.logger.info 'Redis:', 'Install hiredis-client for faster bindings'
end

module Jekyll
  # Replaces Jekyll::Cache to use a Redis store
  class Cache
    # A Marshaled `nil` value
    NIL = "\x04\b0"
    @base_cache = {}
    @disk_cache_enabled = true

    class << self
      attr_accessor :cache_dir
      attr_reader :base_cache, :disk_cache_enabled

      # Do nothing for these methods
      def clear_if_config_changed(_) ; end
      def disable_disk_cache!; end
      def clear; end
    end

    # Initilize a cache by connecting to a Redis server.
    #
    # Cached keys are prefixed with the site's path, so the same server
    # can be shared by other sites.
    def initialize(name)
      @base = "#{Digest::SHA2.hexdigest(cache_dir.to_s)}/#{name}"
      @cache = {}
      @config = RedisClient.config(url: ENV['REDIS_URL'])
      @redis = @config.new_pool(size: 2)
      @redis.call('PING')
      @stats = {}
    end

    # Execs Redis commands and collects stats
    def redis(*args)
      @stats[args.first] ||= { time: 0, bytes: 0, count: 0 }
      @stats[args.first][:count]  += 1
      @redis.call(*args)
    end

    # Redis manages this
    def clear; end

    # Print stats
    def render_table
      LiquidRenderer::Table.new(@stats).to_s(50)
    end

    # Retrieves a key
    #
    # @param :key [Any]
    # @return [Any]
    def [](key)
      hashed_key = hash_key key

      @cache[hashed_key] ||= Marshal.load(redis('GET', hashed_key) || NIL)
    end

    # Stores a Marshaled value indexed by a key.
    #
    # @param :key [Any]
    # @param :value [Any]
    # @return [nil]
    def []=(key, value)
      redis('SET', hash_key(key), Marshal.dump(value))
      nil
    end

    # Returns a key or set its value to a block result
    #
    # @param :key [Any]
    # @return [Any]
    def getset(key, &block)
      self[key] ||= yield
    end

    # Removes a key from Redis
    #
    # @param :key [Any]
    # @return [nil]
    def delete(key)
      redis('DEL', hash_key(key))
      nil
    end

    # Discovers if a key exists
    #
    # @param :key [Any]
    # @return [Boolean]
    def key?(key)
      redis('EXISTS', hash_key(key)) > 0
    end

    # Always true
    #
    # @return [TrueClass]
    def disk_cache_enabled?
      true
    end

    private

    # Creates a unique entry for a given key
    #
    # @param :key [String]
    # @return [String]
    def hash_key(key)
      @hash_keys ||= {}
      @hash_keys[key] ||= "#{@base}/#{Digest::SHA2.hexdigest(key.to_s)}"
    end
  end
end
